# subscription by config
import sets
from twisted.internet import reactor
from twisted.internet.defer import Deferred
from habu import habuutils
from habu.webutils import getPage
import feedparser
import habu.log as log


class FeedConfig(object):
    def __init__(self, feedConfig):
        self.feedList = feedConfig
    
    def fetch(self):
        return self.feedList

class FileConfig(object):
    def __init__(self, filePath):
        self.path = filePath

    def fetch(self):
        lines = open(self.path).readlines()
        return [ l.strip() for l in lines]

class OpmlConfig(object):
    def __init__(self, conf):
        pass

    def fetch(self):
        return []

class Config(object):
    def __init__(self, config, environ):
        self.subModules = []
        self.fetchConfigs = []
        feedConfig = config.setdefault("feed", [])[:]
        if feedConfig:
            self.fetchConfigs.append(FeedConfig(feedConfig))
        feedConfig = config.setdefault("file", None)
        if feedConfig:
            self.fetchConfigss.append(FileConfig(feedConfig))
        self.maxThread = config.setdefault("max-thread", 0)
        self.proxy_host = environ.get("proxy_host", None)
        self.proxy_port = environ.get("proxy_port", 0)

    def setSubModules(self, subModules):
        self.subModules = subModules
        
    def execute(self, content=None):
        feeds = []
        for config in self.fetchConfigs:
            feeds.extend(config.fetch())
        self.feed = sets.Set(feeds)
        try:
            self.feed.remove('')
        except:
            pass

        self.deferredList = [Deferred()  for f in self.feed]

        maxThread = min(
            self.maxThread > 0 and self.maxThread or len(self.feed),
            len(self.feed))

        for i in range(0, maxThread):
            url = self.feed.pop()
            getPage(url, self.proxy_host, self.proxy_port).addCallback(
                self.gotPage).addErrback(
                self.gotErr)

        result = self.deferredList[:]
        result = habuutils.mergeResult(result, self.subModules)

        return result

    def gotErr(self, failure):
        log.debug("filter.config gotErr")
        deferred = self.deferredList.pop()
        deferred.errback(failure)

    def postParse(self, content):
        feed = content["feed"]
        if not feed.has_key("link") and feed.get("id", None):
            feed["link"] = feed.get("id")
        
    def gotPage(self, content):
        log.debug("filter.config gotPage")
        feed = feedparser.parse(content)
        if feed  and not feed.has_key("bozo_exception"):
            self.postParse(feed)
            deferred = self.deferredList.pop()
            deferred.callback(feed)
        else:
            deferred = self.deferredList.pop()
            error_message = "tmiki.subscription.config: parse failed"
            if feed and feed.has_key("bozo_exception"):
                error_message += " " + str(feed.get("bozo_exeception"))
            deferred.errback(error_message)

        if self.feed:
            url = self.feed.pop()
            log.debug("subscription.config : Next Feed Fetcher Start")
            getPage(url, self.proxy_host, self.proxy_port).addCallback(
                self.gotPage).addErrback(
                self.gotErr)

        return content


def create(config, environ):
    return Config(config, environ)
